// This is -*- C++ -*-
// $Id: draw.h,v 1.9 1999/01/07 19:53:26 kenelson Exp $

/* 
 * draw.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_DRAW_H
#define _INC_DRAW_H

#include <gdk--/types.h>
#include <gdk--/drawable.h>
#include <gdk--/gc.h>

// this must be last for gcc 2.7.2 (don't ask why!)
#include <list>


// This is a Gdk_Drawable replacement that's maybe nicer
//  (offers more convenience stuff)
class Gdk_Draw {
public:
  Gdk_Draw();
  Gdk_Draw(Gdk_Drawable&);
  Gdk_Draw(Gdk_Drawable&, Gdk_Drawable&);
  virtual ~Gdk_Draw();

  // If you really want to reach in and fiddle with the underlying things...
  Gdk_Drawable& drawable() { return d1_; }
  Gdk_Drawable& secondary_drawable() { return d2_; }

  /*
    If you were to change between drawables with different visual types
    in midstream, all sorts of bad things could happen with your
    GC stack.  But if you do something that strange, you probably deserve
    whatever happens to you...
  */
  void set_drawable(Gdk_Drawable&);
  void set_secondary_drawable(Gdk_Drawable&);
  void set_drawable(Gdk_Drawable& d1, Gdk_Drawable& d2) {
    set_drawable(d1);
    set_secondary_drawable(d2);
  }

  void set_default_clip_rectangle(gint xo, gint yo, gint w, gint h);

  void set_default_noclip();

  void set_default_clip_offset_from(Gdk_Draw & source, 
				    gint xoffset,
				    gint yoffset);

  gint x_origin() { return xo_; }
  gint y_origin() { return yo_; }
  gint width();
  gint height();

  /*
    
    We provide a "GC Stack", which I've found can make it easier to write
    programs that are well-behaved without lots of excess passing around
    and duplicating of GCs.

    The top of the stack is the "current GC", the default GC that is
    used in drawing operations if another GC isn't specified.

    The relevant class functions are:
    gc() - fetch a pointer to the current Gdk_GC
    gc_save() - push a new Gdk_GC with all of the standard default values
                on top of the stack
    gc_clone() - push a copy of the current Gdk_GC onto the stack
    gc_restore() - pop the current Gdk_GC off of the stack.  The stack can
                   never be empty --- a default Gdk_GC will appear as the
                   current Gdk_GC when you pop away the last element in the
                   stack.
    gc_push() - put a given GC on top of the stack.  You are no longer
                responsible for deleting it... if you do, you'll probably
		have some sort of horrible dangling pointer segfault before
		long.
    gc_pop() - pop the top GC off of the stack, and hand it back to you.
               You are now responsible for deleting it, or you will leak
	       memory!

    In many kinds of drawing routines, I like to be able to call gc_clone()
    when I enter and gc_restore() when I exit.  That way I can change
    drawing colors and such with reckless abandon, while retaining other
    information (like clipping, for example) that might have been set
    by the calling code.

   */

  // By doing this "lazy allocation", we never have to worry about
  // creating the GC before we have been realized.
  // If we've defined a default clipping rectangle, we apply that rectangle
  // to any GC that we create.
  Gdk_GC& gc();
  void gc_save();
  void gc_clone();
  void gc_restore();
  void gc_flush_stack();
  void gc_push(Gdk_GC&);
  Gdk_GC gc_pop();
  
  void draw_point(gint x, gint y) 
    {draw_point(gc(),x,y);}
  void draw_point(Gdk_GC& myGC,gint x, gint y );

  void draw_line(gint x0, gint y0, gint x1, gint y1) 
    {draw_line(gc(),x0,y0,x1,y1);}
  void draw_line(Gdk_GC& myGC,gint x0, gint y0, gint x1, gint y1);

  void draw_points(GdkPoint* points, gint npoints)
    {draw_points(gc(),points,npoints);}
  void draw_points(Gdk_GC& myGC,GdkPoint* points, gint npoints );
  
  void draw_segments(GdkSegment* segs, gint nsegs) 
    {draw_segments(gc(),segs,nsegs);}
  void draw_segments(Gdk_GC& myGC, GdkSegment* segs, gint nsegs );

  void draw_lines(GdkPoint* points, gint npoints)
    {draw_lines(gc(),points,npoints);}
  void draw_lines(Gdk_GC& myGC, GdkPoint* points, gint npoints );

  // draw a rectangle, specifying the (x,y) of the upper left-hand corner
  // and the width and height.
  void draw_rectangle(bool filled, gint x, gint y, gint w, gint h )
    {draw_rectangle(gc(),filled,x,y,w,h);}
  void draw_rectangle(Gdk_GC& myGC,bool filled, gint x, gint y, 
                      gint w, gint h );

  // draw a rectangle, specifying the (x,y) of two of the opposite corners.
  void draw_rectangle_coor(bool filled, gint x0, gint y0, 
                           gint x1, gint y1)
    {draw_rectangle_coor(gc(),filled,x0,y0,x1,y1);}
  void draw_rectangle_coor(Gdk_GC& myGC, bool filled, gint x0, gint y0, 
                           gint x1, gint y1);

  /*
    Annoying X-ism: When specifying the angle for an arc, it has to be given
    as degrees*64.  Therefore an arc that sweeps from 0 to 180 degrees
    should have a1=0, a2=11530 (=180*64).
  */
  void draw_arc(bool filled, gint x, gint y, gint w, gint h, gint a1, gint a2)
    {draw_arc(gc(),filled,x,y,w,h,a1,a2);}
  void draw_arc(Gdk_GC& myGC, bool filled, gint x, gint y, 
                gint w, gint h, gint a1, gint a2);

  // This is usually what I want to do when I call draw_arc() anyway...
  void draw_circle(bool filled, gint x, gint y, guint r)
    {draw_circle(gc(),filled,x,y,r);}
  void draw_circle(Gdk_GC& myGC, bool filled, gint x, gint y, guint r );

  void draw_polygon(bool filled, GdkPoint* points, gint npoints) 
    {draw_polygon(gc(),filled,points,npoints);}
  void draw_polygon(Gdk_GC& myGC, bool filled, GdkPoint* points, gint npoints);

  Gdk_Pixmap copy_rectangle_to_pixmap(gint x, gint y, gint w, gint h);
  Gdk_Pixmap copy_rectangle_to_pixmap(Gdk_GC& myGC, gint x, gint y, 
                                      gint w, gint h);

  void paste_pixmap(Gdk_Pixmap& pix, gint x, gint y);
  void paste_pixmap(Gdk_GC& myGC,Gdk_Pixmap& pix, gint x, gint y );

  void draw_string(gint x, gint y, const string& str)
    {draw_string(gc(),x,y,str);}
  void draw_string(Gdk_GC& myGC, gint x, gint y, const string& str );

  /*
    Normally when drawing text, the lower left corner of the first
    character is placed at the given (x,y) coordinate.  This routine
    lets you control the alignment of the text around the (x,y) coordinate.

    Alignment is specified as follows:
    If x_align < 0, the right edge of the text string is at the given x.
    If x_align > 0, the left edge of the text string is at the given x.
    if x_align = 0, the string is horizontally centered around the given x.

    If y_align < 0, the bottom edge of the text string is at the given y.
    If y_align > 0, the top edge of the text string is at the given y.
    If y_align = 0, the string is vertically centered around the given y.

    x_adjust and y_adjust allows the text to be shifted the given number
    of pixels in the direction of the alignment.  (So if x_align < 0
    and x_adjust = 10, the right edge of the text is set 10 pixels to left
    of the given x.)

  */

  void draw_aligned_string(gint x, gint y, const string& str,
			   gint x_align = 0, gint y_align = 0,
			   gint x_adjust = 0, gint y_adjust = 0)
    {draw_aligned_string(gc(),x,y,str,x_align,y_align,x_adjust,y_adjust);}
  void draw_aligned_string(Gdk_GC& myGC,gint x, gint y, const string& str,
			   gint x_align = 0, gint y_align = 0,
			   gint x_adjust = 0, gint y_adjust = 0);

  void draw_text(gint x, gint y, const string& str, 
                 gint text_align = -1)
    {draw_text(gc(),x,y,str,text_align);}
  void draw_text(Gdk_GC& myGC,gint x, gint y, const string& str, 
                 gint text_align = -1);

#if GDK_VERSION_GT(1,0)
  void draw_aligned_text(gint x, gint y, const string& str,
			 gint text_align = -1,
			 gint x_align = 0, gint y_align = 0,
			 gint x_adjust = 0, gint y_adjust = 0)
    {
     draw_aligned_text(gc(),x,y,str,text_align,x_align,y_align,
                       x_adjust,y_adjust);
    }
  void draw_aligned_text(Gdk_GC& myGC,gint x, gint y, const string& str,
			 gint text_align = -1,
			 gint x_align = 0, gint y_align = 0,
			 gint x_adjust = 0, gint y_adjust = 0);
#endif

private:
  Gdk_Drawable d1_;
  Gdk_Drawable d2_;

  gint xo_, yo_, w_, h_;
  
  Gdk_GC gc_;
  list<Gdk_GC> gc_stack_;
 
};

#endif // _INC_DRAWABLE_H

// $Id: draw.h,v 1.9 1999/01/07 19:53:26 kenelson Exp $
